#!/usr/bin/env python

import os
from os.path            import join, exists
import gtk
from hashlib            import md5

from crebs.files        import create_image_list, make_needed_dirs
from crebs.debug        import report


def get_scaled_duration(duration):
    """Returns the default timings for the GUI."""

    if duration % 3600 == 0:
        duration_scaled = duration / 3600
        scale = 2

    elif duration % 60 == 0:
        duration_scaled = duration / 60
        scale = 1

    else:
        duration_scaled = duration
        scale = 0

    return (duration_scaled, scale)


def resize_thumbnail(pixbuf, target_size = 96, filename = ''):
    """Resizes a thumbnail and, optionally, saves it for later use."""

    aspect_ratio = float(pixbuf.get_width()) / float(pixbuf.get_height())

    if (aspect_ratio >= 1):
        x = target_size
        y = int(target_size / aspect_ratio)
    else:
        y = target_size
        x = int(target_size * aspect_ratio)

    pixbuf = pixbuf.scale_simple(x,y,gtk.gdk.INTERP_BILINEAR)

    if filename:
        make_needed_dirs(filename)
        try:
            pixbuf.save(filename, 'png')
        except:
            report(4, "Unable to save image to .thumbnails cache.", 2)
        else:
            report(4, "Thumbnail cached.", 2)

    return pixbuf


def get_thumbnail(filename, target_size = 96):
    """Returns a thumbnail for the specified image as a Pixbuf."""

    report(4, "Generating thumbnail...", 1)

    # Validate filename as a usable image type
    try:
        f = open(filename, 'r')
        try:
            twisted_filestarter = f.read(512)
        finally:
            f.close()
    except IOError:
        report(4, "Error: Cannot read as a file: {0}".format(filename), 1.5)
        return False
    else:
        if twisted_filestarter[:8] == "\x89\x50\x4e\x47\x0D\x0A\x1A\x0A":
            report(4, "PNG: {0}".format(filename), 2)

        elif twisted_filestarter[:2] == "\xff\xd8":
            report(4, "JPEG: {0}".format(filename), 2)

        elif twisted_filestarter[:6] == 'GIF89a' or twisted_filestarter[:6] == 'GIF87a':
            report(4, "GIF: {0}".format(filename), 2)

        elif 'xml' in twisted_filestarter and 'http://www.w3.org/2000/svg' in twisted_filestarter:
            report(4, "SVG: {0}".format(filename), 2)

        else:
            report(4, "Invalid filetype: {0}".format(filename), 2)
            report(6, twisted_filestarter) # DEBUG data for -vvvvvv
            return False

    # Look in thumbnail cache first (~/.thumbnails/normal)
    thumb_dir = join(os.getenv('HOME'), '.thumbnails', 'normal')
    thumb = join(thumb_dir, md5('file://' + filename).hexdigest() + '.png')

    if not exists(thumb):
        thumbnail_needed = True
    else:
        thumbnail_needed = False
        filename = thumb
        report(4, "Cached thumbnail located.", 1.5)

    # Generate correctly sized thumbnail
    try:
        pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
    except:
        report(4, "Unable to create pixbuf.", 1.5)
        return False
    else:
        if thumbnail_needed:
            # Save 128-pixel thumbnail to cache
            pixbuf = resize_thumbnail(pixbuf, 128, thumb)

        resized = resize_thumbnail(pixbuf, target_size)

        return resized



def convert_to_gui_image_list(basic_list, def_duration, def_transition, gui_list = []):
    """Converts a basic image list into a GUI image list."""

    if not gui_list:
        # Image list: filename; icon; use def. dur.; dur./scale; scale; use def. trans.; trans
        gui_list = gtk.ListStore(str, gtk.gdk.Pixbuf, bool, int, int, bool, float)

    for image in basic_list:
        filename   = image[0]
        duration   = image[1]
        transition = image[2]

        icon = get_thumbnail(filename)

        if not icon:
            report(3, "This file is not a valid image.", 1)
            report(3, filename, 2)
        else:
            report(2, _("Adding an image..."), 1)
            report(2, filename, 2)

            if duration == def_duration or duration < 0:
                use_def_dur = True
                duration = def_duration
            else:
                use_def_dur = False

            if transition == def_transition or transition < 0:
                use_def_trn = True
                transition = def_transition
            else:
                use_def_trn = False

            (dur_scaled, scale) = get_scaled_duration(duration)

            gui_list.append((filename, icon, use_def_dur, dur_scaled, scale, use_def_trn, transition))

            report(3, "Success.", 1.5)

    return gui_list


def create_gui_image_list(files, options):
    """Returns the image list used by the GUI, initialised with invocation arguments."""

    # Get any images passed as arguments
    inv_image_list = create_image_list(files, options, 1)

    # Convert to GUI list store
    gui_image_list = convert_to_gui_image_list(inv_image_list, options.default_duration, options.default_transition)

    return gui_image_list



